/**
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * beck et al. projects GmbH
 * Theresienhoehe 13
 * 80336 Munich
 * Phone: +49-89-5442530
 * mailto: info@bea.de
 */
package de.bsvrz.ars.export;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.Window;

/**
 * Statische Hilfsklasse zum Positionieren von Fenstern
 * 
 * @author beck et al. projects GmbH
 * @author Phil Schrettenbrunner
 * @version $Revision: 1.1 $ / $Date: 2008/01/22 16:39:42 $ / ($Author: yvonnes $)
 */
public class Positioner {
    /**
     * Nur statische Methoden, keine Instanzen... 
     */
    private Positioner() {}


    /**
     * Hilfsmethode, die ein Fenster zentriert darstellt. Dabei kann als
     * Bezugspunkt entweder ein anderes Fenster oder der Desktop verwendet
     * werden
     * 
     * @param parent
     *            Der Bezugspunkt. Wenn null, ist der Desktop der Bezugspunkt
     * @param myself
     *            Das zu positionierende Windoe
     */
    public static void center(Window parent, Window myself) {
        Dimension d = null;
        Point p = null;

        if (parent != null) {
            d = parent.getSize();
            p = parent.getLocation();
        } else {
            d = Toolkit.getDefaultToolkit().getScreenSize();
            p = new Point();
        }

        double centreX = p.getX() + d.getWidth() / 2;
        double centreY = p.getY() + d.getHeight() / 2;

        myself.getSize(d);
        p.setLocation(centreX - d.getWidth() / 2, centreY - d.getHeight() / 2);
        if (p.getX() < 0)
            p.setLocation(0, p.getY());

        if (p.getY() < 0)
            p.setLocation(p.getX(), 0);

        myself.setLocation(p);
    }
}
