/**
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * beck et al. projects GmbH
 * Theresienhoehe 13
 * 80336 Munich
 * Phone: +49-89-5442530
 * mailto: info@bea.de
 */
package de.bsvrz.ars.export.prot;

import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JOptionPane;

import de.bsvrz.ars.export.Dataspecification;
import de.bsvrz.ars.export.DataspecificationList;
import de.bsvrz.ars.export.ExportProfile;
import de.bsvrz.ars.export.ExportProfile.Format;
import de.bsvrz.ars.export.interfaces.ProgressListener;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.archive.ArchiveData;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKindCombination;
import de.bsvrz.dav.daf.main.archive.ArchiveDataQueryResult;
import de.bsvrz.dav.daf.main.archive.ArchiveDataSpecification;
import de.bsvrz.dav.daf.main.archive.ArchiveDataStream;
import de.bsvrz.dav.daf.main.archive.ArchiveOrder;
import de.bsvrz.dav.daf.main.archive.ArchiveQueryPriority;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestManager;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestOption;
import de.bsvrz.dav.daf.main.archive.ArchiveTimeSpecification;
import de.bsvrz.dav.daf.main.archive.TimingType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.pat.onlprot.protocoller.protocolModuleConnector.ProtocolModuleConnector;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Ein Adapter, der zwischen einen Stream vom Archivsystem und einem Protokollierungsmodul steht und
 * die von der take() Methode kommenden Daten umwandelt und mit ihnen die update() Methode aufruft.
 * 
 * @author beck et al. projects GmbH
 * @author Phil Schrettenbrunner
 * @version $Revision: 1.2 $ / $Date: 2008/05/22 16:21:19 $ / ($Author: martinh $)
 */
public class ProtocollerAdapter extends Thread {
    public final int ARC_MAX_WAIT = 5;
    
    /**
     * 
     */
    public static final String SQL_MODULE = "-protModul=de.bsvrz.ars.export.prot.DatabaseProtocoller";
    /**
     * 
     */
    public static final String XML_MODULE = "-protModul=de.bsvrz.pat.onlprot.standardProtocolModule.StandardProtocoller";

    /**
     * 
     */
    public static final String LINE_MODULE = "-protModul=de.bsvrz.ars.export.prot.FlatProtocoller";

    private ExportProfile exportProfile;
    private ProgressListener progressListener;
    private ClientDavInterface connection;
    private ProtocolModuleConnector pmc;
    private boolean isReady = false;
    private boolean terminate = false;


    /**
     * Erzeugt eine neue Instanz eines Protokoll Adapters
     * 
     * @param exportProfile Objekt, dass die Informatinen zum Datenexport enthlt
     * @param progressListener Objekt, das sich fr den Fortschritt des Exports interessiert
     */
    public ProtocollerAdapter(ExportProfile exportProfile, ProgressListener progressListener) {
        prepare(exportProfile, progressListener);
    }


    /**
     * Erzeugt eine neue Instanz eines Protokoll Adapters, die sich nicht fr den Fortschritt
     * interessiert
     * 
     * @param exportProfile Objekt, dass die Informatinen zum Datenexport enthlt
     */
    public ProtocollerAdapter(ExportProfile exportProfile) {
        prepare(exportProfile, new ProgressListener() {
            public void init(ProtocollerAdapter pa) {
            }


            public void done() {
            }


            public void recordReceived() {
            }


            public void setDSpecTotal(int total) {

            }


            public void setFormat(String s) {
            }


            public void nextDataSpecification(int counter, int max, boolean resetCounter) {
            }
        });
    }


    /**
     * @param exportProfile
     * @param progressListener
     * 
     */
    private void prepare(ExportProfile exportProfile, ProgressListener progressListener) {
        this.exportProfile = exportProfile;
        this.connection = exportProfile.getDavConnection();
        this.progressListener = progressListener;

        // Eine ArgumentList erzeugen
        String[] arg = new String[3];

        if (exportProfile.getFormat() == Format.SQL) {
            arg[0] = SQL_MODULE;
            arg[2] = "";
        } else if (exportProfile.getFormat() == Format.XML) {
            arg[0] = XML_MODULE;
            arg[2] = "-ausgabe=xml";
        } else if (exportProfile.getFormat() == Format.LINE) {
            arg[0] = LINE_MODULE;
            arg[2] = "-ausgabe=line";
        } else {
            isReady = false;
        }

        if (exportProfile.getOutputFile() != null) {
            arg[1] = "-datei=" + exportProfile.getOutputFile();
        }

        ArgumentList argList = new ArgumentList(arg);

        pmc = null;
        try {
            pmc = new ProtocolModuleConnector(argList, arg);
            isReady = true;
        } catch (ClassNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            isReady = false;
        } catch (IllegalAccessException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            isReady = false;
        } catch (InstantiationException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            isReady = false;
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            isReady = false;
        }
    }


    /**
     * @return Wahr, wenn der Adapter bereit ist und ein Protokollierungs Modul geladen hat.
     */
    public boolean isReady() {
        return isReady;
    }


    /**
     * Nachtrgliches Setzen eines Objektes, das sich fr den Fortschritt interessiert
     * 
     * @param progressListener Objekt, das ber den Exportvortschritt informiert wird/werden soll
     */
    public void setProgressListener(ProgressListener progressListener) {
        this.progressListener = progressListener;
    }


    /**
     * Einstiegspunkt fr den Export
     * 
     * @see java.lang.Runnable#run()
     */
    public void run() {
        assert progressListener != null : "Kein Progresslistener";

        progressListener.setFormat(exportProfile.getFormat().toString());
        ClientReceiverInterface protocoller = pmc.getProtocoller();

        // /// Einstellungen aus Eingabe in Objekte umwandeln
        ArchiveQueryPriority archiveQueryPriority = ArchiveQueryPriority.MEDIUM;
        ArchiveTimeSpecification archiveTimeSpecification = new ArchiveTimeSpecification(TimingType.DATA_TIME, false, exportProfile.getFrom(), exportProfile
                .getUntil());
        ArchiveDataKindCombination archiveDataKindCombination = new ArchiveDataKindCombination(ArchiveDataKind.ONLINE, ArchiveDataKind.ONLINE_DELAYED,
                ArchiveDataKind.REQUESTED, ArchiveDataKind.REQUESTED_DELAYED);
        ArchiveOrder archiveOrder = ArchiveOrder.BY_DATA_TIME;
        ArchiveRequestOption archiveRequestOption = ArchiveRequestOption.NORMAL;

        // /// Variablen deklarieren
        ArchiveDataStream[] archiveDataStream = null;

        // /// Export durchfhren
        try {
            ArchiveRequestManager archive = connection.getArchive();
            if (terminate) {
                endExport(archiveDataStream);
                return;
            }

            int cnt = 0;
            while(!archive.isArchiveAvailable()) {
                synchronized (this) { this.wait(1000); }
                Debug.getLogger().fine("Warte auf das Archivsystem. Versuch " + cnt + " von " + ARC_MAX_WAIT);
                if (cnt>=ARC_MAX_WAIT) {
                    if (exportProfile.openGui()) {
                        JOptionPane.showMessageDialog(null,
                                "Es konnte keine Verbindung zum Archivsystem aufgebaut werden.\nBitte versuchen Sie es spter nocheinmal.\n\n"
                                        + "Auf der Konsole stehen mglicherweise weitere Informationen.", "Archivsystem steht nicht zur Verfgung",
                                JOptionPane.INFORMATION_MESSAGE);
                        progressListener.done();
                    } else {
                        Debug.getLogger().error("Es konnte keine Verbindung zum Archivsystem aufgebaut werden.");
                    }
                    return; // Kein ArS -> Ende
                }
                cnt++;
            }

            // /// Die eigentliche Anfrage zusammenstellen
            DataspecificationList dsl = exportProfile.getDataSpecificationList();
            for (int i = 0; i < dsl.getSize(); i++) {
                progressListener.nextDataSpecification(i + 1, dsl.getSize(), true);

                Dataspecification ds = (Dataspecification) dsl.getElementAt(i);
                Collection<SystemObject> sysObjects = ds.getSystemObjects();

                DataDescription dataDescription = new DataDescription(ds.getAtg(), ds.getAsp(), ds.getSimVar());
                List<ArchiveDataSpecification> archiveDataSpecifications = new LinkedList<ArchiveDataSpecification>();
                for (Iterator iterator = sysObjects.iterator(); iterator.hasNext();) {
                    SystemObject sysObject = (SystemObject) iterator.next();
                    archiveDataSpecifications.add(new ArchiveDataSpecification(archiveTimeSpecification, archiveDataKindCombination, archiveOrder,
                            archiveRequestOption, dataDescription, sysObject));
                }

                ArchiveDataQueryResult queryResult = archive.request(archiveQueryPriority, archiveDataSpecifications);

                if (queryResult.isRequestSuccessful()) {
                    if (terminate) {
                        endExport(archiveDataStream);
                        return;
                    }

                    archiveDataStream = queryResult.getStreams();

                    for (ArchiveDataStream stream : archiveDataStream) {
                        ArchiveData ad = stream.take();
                        while (ad != null && !terminate) {
                            if (ad.getDataType() != DataState.END_OF_ARCHIVE) {
                                ResultData rd[] = new ResultData[1];
                                rd[0] = new ResultData(ad.getObject(), ad.getDataDescription(), ad.getDataTime(), ad.getData());
                                protocoller.update(rd);
                                progressListener.recordReceived();
                            }
                            ad = stream.take();
                        }
                    }
                } else {
                    System.out.println("Eine Archivanfrage konnte nicht bearbeitet werden, Fehler: " + queryResult.getErrorMessage());
                }

            } // for-Schleife
        } catch (Exception e) {
            e.printStackTrace();
        }

        // Kurze Pause, bevor sich das Fenster schliet
        synchronized (protocoller) {
            try {
                protocoller.wait(750);
            } catch (InterruptedException e) {
            } // Ignorieren
        }

        endExport(archiveDataStream);
    }


    /**
     * 
     */
    public void terminate() {
        terminate = true;
    }


    private void endExport(ArchiveDataStream[] archiveDataStream) {
        if (archiveDataStream != null)
            for (ArchiveDataStream stream : archiveDataStream)
                stream.abort();

        pmc.cleanUp();

        progressListener.done();
    }
}
